const gulp = require('gulp');
const autoprefixer = require('gulp-autoprefixer');
const cleanCss = require('gulp-clean-css');
const concat = require('gulp-concat');
const eslint = require('gulp-eslint');
const fs = require('fs');
const livereload = require('gulp-livereload');
const magicImporter = require('node-sass-magic-importer');
const path = require('path');
const plumber = require('gulp-plumber');
const sass = require('gulp-sass');
const sassLint = require('gulp-sass-lint');
const uglify = require('gulp-uglify');
const watch = require('gulp-watch');
const webpack = require('webpack-stream');

var config = {
	package: JSON.parse(fs.readFileSync('package.json')),
	paths: {
		style: ['node_modules']
	}
};

var files = {
    scripts: ['src/*.js'],
    styles: ['src/*.scss']
};

if ('dependencies' in config.package && 'aitomic-base' in config.package.dependencies) {
	config.lint = { style: 'node_modules/aitomic-base/config-sass-lint.yml', es: require('./node_modules/aitomic-base/es-lint').get };

	config.paths.style = ['node_modules', 'node_modules/aitomic-base/src'];

	files = {
		scripts: ['node_modules/aitomic-base/src/index.js'].concat(files.scripts),
		styles: ['node_modules/aitomic-base/src/begin.scss'].concat(files.styles, ['node_modules/aitomic-base/src/end.scss'])
	};
}

gulp.task('default', function() {
	gulp.start('styles');
	gulp.start('scripts');
});

gulp.task('styles', ['styles-lint'], function() {
	if (files.styles.length == 0) return;

	return gulp.src(files.styles)
			.pipe(concat('style.css'))
			.pipe(plumber({
				errorHandler: function(err) {
					console.log(err);
					this.emit('end');
				}
			}))
			.pipe(sass({
				includePaths: config.paths.style || [],
				importer: [
					function (url, prev, done) {
						if (url.indexOf('~') == 0 && path.extname(url) === '.css' &&
							fs.existsSync(nodeUrl = 'node_modules/' + url.replace('~', '')))
							return done({ contents: fs.readFileSync(nodeUrl, 'utf8') });
						return null;
					},
					magicImporter()
				]
			}))
			.pipe(autoprefixer())
			.pipe(cleanCss({
				compatibility: 'ie10',
				inline: ['local']
			}))
			.pipe(gulp.dest('dist'))
			.pipe(livereload());
});

gulp.task('styles-lint', function() {
	return gulp.src('src/*.scss')
			.pipe(sassLint({
				configFile: config.lint.style || ''
			}))
			.pipe(sassLint.format())
			.pipe(sassLint.failOnError());
});

gulp.task('scripts', ['scripts-lint'], function() {
	if (files.scripts.length == 0) return;

	return gulp.src(files.scripts)
			.pipe(concat('script.js'))
			.pipe(gulp.dest('dist'))
			.pipe(webpack({
				output: {
					filename: 'script.js'
				},
				mode: 'production',
				module: {
					rules: [
						{
							use: {
								loader: 'babel-loader',
								options: {
									presets: ['@babel/preset-env']
								}
							}
						}
					]
				}
			}))
			.pipe(uglify())
			.pipe(gulp.dest('dist'))
			.pipe(livereload());
});

gulp.task('scripts-lint', function() {
	return gulp.src('src/*.js')
		.pipe(eslint(config.lint.es || ''))
		.pipe(eslint.format())
		.pipe(eslint.failAfterError());
});

gulp.task('watch', function() {
	gulp.start('styles');
	gulp.start('scripts');

	watch('src/*.scss', { usePolling: true }, function() {
		gulp.start('styles');
	});

	watch('src/*.js', { usePolling: true }, function() {
		gulp.start('scripts');
	});

	gulp.watch(['src/*.latte', 'src/*.json']).on('change', function() {
		livereload.reload();
	});

	livereload.listen();
});

module.exports = gulp;